@@ -433,7 +433,7 @@ static dberr_t srv_undo_delete_old_tablespaces()
 /** Recreate the undo log tablespaces */
 ATTRIBUTE_COLD static dberr_t srv_undo_tablespaces_reinit()
 {
-  mtr_t mtr{nullptr};
+  mtr_t mtr;
   dberr_t err;
   buf_block_t *first_rseg_hdr;
   uint32_t latest_space_id;
@@ -626,7 +626,7 @@ static dberr_t srv_undo_tablespaces_reinitialize()
 static uint32_t trx_rseg_get_n_undo_tablespaces()
 {
   std::set<uint32_t> space_ids;
-  mtr_t mtr{nullptr};
+  mtr_t mtr;
   mtr.start();
 
   if (const buf_block_t *sys_header=
@@ -879,8 +879,6 @@ unused_undo:
   return DB_SUCCESS;
 }
 
-PRAGMA_DISABLE_CHECK_STACK_FRAME
-
 /** Open the configured number of dedicated undo tablespaces.
 @param[in]	create_new_undo	whether the undo tablespaces has to be created
 @param[in,out]	mtr		mini-transaction
@@ -942,9 +940,6 @@ dberr_t srv_undo_tablespaces_init(bool create_new_undo, mtr_t *mtr)
   return err;
 }
 
-PRAGMA_REENABLE_CHECK_STACK_FRAME
-
-
 /** Create the temporary file tablespace.
 @param[in]	create_new_db	whether we are creating a new database
 @return DB_SUCCESS or error code. */
@@ -979,7 +974,7 @@ srv_open_tmp_tablespace(bool create_new_db)
 		ib::error() << "Unable to create the shared innodb_temporary";
 	} else if (fil_system.temp_space->open(true)) {
 		/* Initialize the header page */
-		mtr_t mtr{nullptr};
+		mtr_t mtr;
 		mtr.start();
 		mtr.set_log_mode(MTR_LOG_NO_REDO);
 		err = fsp_header_init(fil_system.temp_space,
@@ -1000,19 +995,12 @@ srv_open_tmp_tablespace(bool create_new_db)
 	return(err);
 }
 
-/** Shutdown background threads, except the page cleaner.
-@param init_abort set to true when InnoDB startup aborted */
-static void srv_shutdown_threads(bool init_abort= false)
+/** Shutdown background threads, except the page cleaner. */
+static void srv_shutdown_threads()
 {
 	ut_ad(!srv_undo_sources);
 	srv_master_timer.reset();
-	/* In case of InnoDB start up aborted, Don't change
-	the srv_shutdown_state. Because innodb_shutdown()
-	does call innodb_preshutdown() which changes the
-	srv_shutdown_state back to SRV_SHUTDOWN_INITIATED */
-	if (!init_abort) {
-		srv_shutdown_state = SRV_SHUTDOWN_EXIT_THREADS;
-	}
+	srv_shutdown_state = SRV_SHUTDOWN_EXIT_THREADS;
 
 	if (purge_sys.enabled()) {
 		srv_purge_shutdown();
@@ -1082,7 +1070,7 @@ srv_init_abort_low(
 	}
 
 	srv_shutdown_bg_undo_sources();
-	srv_shutdown_threads(true);
+	srv_shutdown_threads();
 	return(err);
 }
 
@@ -1213,7 +1201,7 @@ static dberr_t srv_log_rebuild_if_needed()
 
   if (log_sys.file_size == srv_log_file_size &&
       log_sys.format ==
-      (srv_encrypt_log ? log_t::FORMAT_ENC_11 : log_t::FORMAT_10_8))
+      (srv_encrypt_log ? log_t::FORMAT_ENC_10_8 : log_t::FORMAT_10_8))
   {
     /* No need to add or remove encryption, upgrade, or resize. */
     delete_log_files();
@@ -1253,54 +1241,13 @@ inline lsn_t log_t::init_lsn() noexcept
   return lsn;
 }
 
-PRAGMA_DISABLE_CHECK_STACK_FRAME
-
-/** Load the dictionary tables */
-static dberr_t srv_load_tables(bool must_upgrade_ibuf) noexcept
-{
-  mem_heap_t *heap= mem_heap_create(1000);
-  mtr_t mtr{nullptr};
-  dict_sys.lock(SRW_LOCK_CALL);
-  dberr_t err = dict_load_indexes(&mtr, dict_sys.sys_tables, false, heap,
-                                  DICT_ERR_IGNORE_NONE);
-  mem_heap_empty(heap);
-  if ((err == DB_SUCCESS || srv_force_recovery >= SRV_FORCE_NO_DDL_UNDO) &&
-      UNIV_UNLIKELY(must_upgrade_ibuf))
-  {
-    dict_sys.unlock();
-    dict_load_tablespaces(nullptr, true);
-    err= ibuf_upgrade();
-    dict_sys.lock(SRW_LOCK_CALL);
-  }
-  if (err == DB_SUCCESS || srv_force_recovery >= SRV_FORCE_NO_DDL_UNDO)
-  {
-    err = dict_load_indexes(&mtr, dict_sys.sys_columns, false, heap,
-                            DICT_ERR_IGNORE_NONE);
-    mem_heap_empty(heap);
-  }
-  if (err == DB_SUCCESS || srv_force_recovery >= SRV_FORCE_NO_DDL_UNDO)
-  {
-    err = dict_load_indexes(&mtr, dict_sys.sys_indexes, false, heap,
-                            DICT_ERR_IGNORE_NONE);
-    mem_heap_empty(heap);
-  }
-  if (err == DB_SUCCESS || srv_force_recovery >= SRV_FORCE_NO_DDL_UNDO) {
-    err = dict_load_indexes(&mtr, dict_sys.sys_fields, false, heap,
-                            DICT_ERR_IGNORE_NONE);
-  }
-  mem_heap_free(heap);
-  dict_sys.unlock();
-  dict_sys.load_sys_tables();
-  return err;
-}
-
 /** Start InnoDB.
 @param[in]	create_new_db	whether to create a new database
 @return DB_SUCCESS or error code */
 dberr_t srv_start(bool create_new_db)
 {
 	dberr_t		err		= DB_SUCCESS;
-	mtr_t		mtr{nullptr};
+	mtr_t		mtr;
 
 	ut_ad(srv_operation <= SRV_OPERATION_RESTORE_EXPORT
 	      || srv_operation == SRV_OPERATION_RESTORE
@@ -1421,9 +1368,22 @@ dberr_t srv_start(bool create_new_db)
 	}
 
 	if (os_aio_init()) {
+		ib::error() << "Cannot initialize AIO sub-system";
+
 		return(srv_init_abort(DB_ERROR));
 	}
 
+#ifdef LINUX_NATIVE_AIO
+	if (srv_use_native_aio) {
+		ib::info() << "Using Linux native AIO";
+	}
+#endif
+#ifdef HAVE_URING
+	if (srv_use_native_aio) {
+		ib::info() << "Using liburing";
+	}
+#endif
+
 	fil_system.create(srv_file_per_table ? 50000 : 5000);
 
 	if (buf_pool.create()) {
@@ -1513,6 +1473,7 @@ dberr_t srv_start(bool create_new_db)
 
 
 	if (err == DB_SUCCESS) {
+		mtr_t mtr;
 		mtr.start();
 		err= srv_undo_tablespaces_init(create_new_db, &mtr);
 		mtr.commit();
@@ -1674,11 +1635,25 @@ dberr_t srv_start(bool create_new_db)
 			DBUG_PRINT("ib_log", ("apply completed"));
 
 			if (srv_operation != SRV_OPERATION_RESTORE) {
-				err = srv_load_tables(must_upgrade_ibuf);
-				if (err != DB_SUCCESS) {
-					return srv_init_abort(err);
+				dict_sys.lock(SRW_LOCK_CALL);
+				dict_load_sys_table(dict_sys.sys_tables);
+				dict_sys.unlock();
+
+				if (UNIV_UNLIKELY(must_upgrade_ibuf)) {
+					dict_load_tablespaces(nullptr, true);
+					err = ibuf_upgrade();
+					if (err != DB_SUCCESS) {
+						return srv_init_abort(err);
+					}
 				}
 
+				dict_sys.lock(SRW_LOCK_CALL);
+				dict_load_sys_table(dict_sys.sys_columns);
+				dict_load_sys_table(dict_sys.sys_indexes);
+				dict_load_sys_table(dict_sys.sys_fields);
+				dict_sys.unlock();
+				dict_sys.load_sys_tables();
+
 				err = trx_lists_init_at_db_start();
 				if (err != DB_SUCCESS) {
 					return srv_init_abort(err);
@@ -2032,8 +2007,6 @@ skip_monitors:
 	return(DB_SUCCESS);
 }
 
-PRAGMA_REENABLE_CHECK_STACK_FRAME
-
 /**
   Shutdown purge to make sure that there is no possibility that we call any
   plugin code (e.g., audit) inside virtual column computation.
